# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/4/6


class LazyProxy:
    def __init__(self, cls, *args, **kwargs):
        # print("INIT PROXY")
        self.__dict__['_cls'] = cls
        self.__dict__['_params'] = args
        self.__dict__['_kwargs'] = kwargs
        self.__dict__['_obj'] = None

    def __getattr__(self, item):
        if self.__dict__['_obj'] is None:
            self._init_obj()
        return getattr(self.__dict__['_obj'], item)

    def __setattr__(self, key, value):
        if self.__dict__['_obj'] is None:
            self._init_obj()

        setattr(self.__dict__['_obj'], key, value)

    def _init_obj(self):
        self.__dict__['_obj'] = object.__new__(self.__dict__['_cls'])
        self.__dict__['_obj'].__init__(*self.__dict__['_params'], **self.__dict__['_kwargs'])


class LazyInit:
    def __new__(cls, *args, **kwargs):
        return LazyProxy(cls, *args, **kwargs)



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/2/27
import os
from config import local_conf_loader


class Config:
    SECRET_KEY = local_conf_loader('SECRET_KEY', '*y=ill@it19yx=s(w7lx&jbk7dm3p+o^&%_v%yeao9@a=o7&c4')



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/15
import re
import inspect
from flask_restx import (
    Resource,
    Namespace
)
from yqn_project_cli.utils import merge_iterable_objs


class DANamespace(Namespace):
    def __init__(self, name, description=None, path=None, decorators=None,
                 validate=None, authorizations=None, ordered=False, **kwargs):
        super(DANamespace, self).__init__(name, description=description, path=path, decorators=decorators,
                                          validate=validate, authorizations=authorizations, ordered=ordered, **kwargs)

        self.error_handlers[BaseException] = self.default_error_deposit

    @staticmethod
    def default_error_deposit(error):
        raise error

    def route(self, *urls, **kwargs):
        def wrapper(cls):
            if inspect.isfunction(cls):
                name = ''.join(map(lambda _str: _str.capitalize(), re.split(r'\.|_', cls.__qualname__)))
                attrs = {cls.__name__: cls}
                for method in kwargs.get('methods', []):
                    attrs[str(method).lower()] = cls
                cls = type(name, (Resource,), attrs)

            doc = kwargs.pop("doc", None)
            if doc is not None:
                kwargs["route_doc"] = self._build_doc(cls, doc)
            self.add_resource(cls, *urls, **kwargs)
            return cls

        return wrapper


def add_view_mth_resource(ns, view_mth, route):
    if inspect.isfunction(view_mth):
        name = ''.join(map(lambda _str: _str.capitalize(), re.split(r'\.|_', view_mth.__qualname__)))
        attrs = {view_mth.__name__: view_mth}
        for method in route.get('http_methods', []):
            attrs[str(method).lower()] = view_mth
        cls = type(name, (Resource,), attrs)

    else:
        raise TypeError('%s must a function' % view_mth)

    doc = route.pop("doc", None)
    if doc is not None:
        route["route_doc"] = ns._build_doc(cls, doc)

    ns.add_resource(cls, *merge_iterable_objs(route['path']), methods=route.pop('http_methods'),
                    **route.pop('path_kwargs', {}))

    return cls



# -*- coding: utf-8 -*-
# Author: ZKH
# Date：2021/3/12
import time
import functools
from flask import request


def parse_request_with(parser):
    def wrapper(view_func):
        @functools.wraps(view_func)
        def inner(*args, **kwargs):
            request.kwargs = parser.parse_args()
            return view_func(*args, **kwargs)

        return inner

    return wrapper


def time_it(timeout=None):
    def wrapper(func):
        @functools.wraps(func)
        def inner(*args, **kwargs):
            start = time.time()
            caller = func(*args, **kwargs)
            waste = time.time() - start
            if timeout is None or (isinstance(timeout, (int, float)) and timeout < waste):
                print(func.__name__, 'waste time: %s sec' % waste)

            return caller

        return inner

    return wrapper
